package com.caimei365.manager.service.wechat.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.caimei365.manager.dao.SystemDao;
import com.caimei365.manager.dao.WeChatDao;
import com.caimei365.manager.entity.PaginationVo;
import com.caimei365.manager.entity.ResponseJson;
import com.caimei365.manager.entity.wechat.WechatArticle;
import com.caimei365.manager.entity.wechat.WechatArticleDetail;
import com.caimei365.manager.service.wechat.WechatArticleService;
import com.github.pagehelper.PageHelper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

import static com.alibaba.fastjson.JSON.parseArray;

/**
 * Description
 *
 * @author : Charles
 * @date : 2022/1/6
 */
@Slf4j
@Service
public class WechatArticleServiceImpl implements WechatArticleService {

    @Value("${wechat.caimei.id}")
    private String caimeiId;
    @Value("${wechat.hehe.id}")
    private String heheId;

    @Resource
    private WeChatDao weChatDao;
    @Resource
    private SystemDao systemDao;

    /**
     * 获取图文素材列表
     *
     * @param type     类型： 1采美，2呵呵商城
     * @param title    标题
     * @param pageNum  页码
     * @param pageSize 每页大小
     */
    @Override
    public ResponseJson<PaginationVo<WechatArticle>> getArticleList(Integer type, String title, int pageNum, int pageSize) {
        String wxType = "";
        if (null != type && 1 == type) {
            wxType = caimeiId;
        } else if (null != type && 2 == type) {
            wxType = heheId;
        }
        PageHelper.startPage(pageNum, pageSize);
        List<WechatArticle> articleList = weChatDao.getArticleList(wxType, title);
        PaginationVo<WechatArticle> pageData = new PaginationVo<>(articleList);
        return ResponseJson.success(pageData);
    }

    /**
     * 根据ID获取图文素材
     */
    @Override
    public ResponseJson<WechatArticle> getArticle(Integer id) {
        WechatArticle article = weChatDao.getWechatArticle(id);
        List<WechatArticleDetail> detailList = weChatDao.getArticleDetailList(article.getId());
        article.setDetailList(detailList);
        return ResponseJson.success(article);
    }

    /**
     * 根据ID更新图文素材
     */
    @Override
    public ResponseJson<Void> updateArticle(Integer id, WechatArticle article, String username) {
        WechatArticle dbArticle = weChatDao.getWechatArticle(id);
        if (null == dbArticle) {
            return ResponseJson.error("当前回复配置异常！", null);
        }
        if (StringUtils.hasLength(article.getTitle())){
            dbArticle.setTitle(article.getTitle());
        }
        if (null != article.getType() && 1 == article.getType()) {
            dbArticle.setWxType(caimeiId);
        } else if (null != article.getType() && 2 == article.getType()) {
            dbArticle.setWxType(heheId);
        }
        Integer userId = systemDao.getUserIdByUsername(username);
        dbArticle.setUserId(userId);
        // 更新
        weChatDao.updateWechatArticle(dbArticle);
        // 文章列表JSON数据解析
        ResponseJson<Void> error = parseArticleJson(dbArticle, article.getDetailJson());
        if (error != null) {
            return error;
        }
        return ResponseJson.success();
    }

    /**
     * 文章列表JSON数据解析
     * @param article 图文素材
     * @param jsonStr 文章列表JSON串
     */
    private ResponseJson<Void> parseArticleJson(WechatArticle article, String jsonStr) {
        // 文章列表JSON数据解析
        JSONArray articleJson = null;
        List<WechatArticleDetail> detailList = new ArrayList<>();
        try {
            articleJson = parseArray(jsonStr);
            if (null == articleJson || articleJson.isEmpty()) {
                return ResponseJson.error("文章列表JSON数据异常！", null);
            }
            for (Object articleObject : articleJson) {
                JSONObject detail = (JSONObject) articleObject;
                Integer articleId = (Integer) detail.get("id");
                String articleTitle = (String) detail.get("title");
                String url = (String) detail.get("url");
                String picurl = (String) detail.get("picurl");
                WechatArticleDetail dbDetail = null;
                if (null != articleId && articleId > 0) {
                    dbDetail = weChatDao.getWechatArticleDetail(articleId);
                } else {
                    dbDetail = new WechatArticleDetail();
                }
                if (StringUtils.hasLength(articleTitle)){
                    dbDetail.setTitle(articleTitle);
                }
                if (StringUtils.hasLength(url)){
                    dbDetail.setUrl(url);
                }
                if (StringUtils.hasLength(picurl)){
                    dbDetail.setPicurl(picurl);
                }
                dbDetail.setArticleId(article.getId());
                dbDetail.setUserId(article.getUserId());
                detailList.add(dbDetail);
            }
        } catch (Exception e) {
            log.error("文章列表JSON数据解析异常try-catch:", e);
            return ResponseJson.error("文章列表JSON数据解析异常！", null);
        }
        List<Integer> dbDetailIds = weChatDao.getArticleDetailIds(article.getId());
        List<Integer> detailIds = new ArrayList<>();
        for (WechatArticleDetail detail : detailList) {
            if (null != detail.getId() && detail.getId() > 0) {
                weChatDao.updateWechatArticleDetail(detail);
                detailIds.add(detail.getId());
            } else {
                weChatDao.insertWechatArticleDetail(detail);
            }
        }
        for (Integer id : dbDetailIds) {
            if (!detailIds.contains(id)){
                weChatDao.deleteWechatArticleDetail(id);
            }
        }
        return null;
    }

    /**
     * 添加图文素材
     */
    @Override
    public ResponseJson<Void> addArticle(WechatArticle article, String username) {
        if (null != article.getType() && 1 == article.getType()) {
            article.setWxType(caimeiId);
        } else if (null != article.getType() && 2 == article.getType()) {
            article.setWxType(heheId);
        } else {
            return ResponseJson.error("配置类型type不正确！", null);
        }
        Integer userId = systemDao.getUserIdByUsername(username);
        article.setUserId(userId);
        // 新增
        weChatDao.insertWechatArticle(article);
        // 文章列表JSON数据解析
        ResponseJson<Void> error = parseArticleJson(article, article.getDetailJson());
        if (error != null) {
            return error;
        }
        return ResponseJson.success();
    }

    /**
     * 根据ID删除图文素材
     */
    @Override
    public ResponseJson<Void> deleteArticle(Integer id) {
        weChatDao.deleteWechatArticle(id);
        weChatDao.deleteDetailByArticleId(id);
        return ResponseJson.success();
    }
}
